package jmaki.runtime;

import javax.servlet.ServletContext;
import java.net.URL;
import java.net.MalformedURLException;
import java.util.HashMap;

/**
 * @author Norbert von Truchsess
 *
 */
public class ResourceManager {
	
	private ServletContext context;
	private HashMap webResources;
	private HashMap classPathResources;
	private boolean usecache;
	private String resoucesDir;
	
	private static final String CLASSPATHPREFIX="/META-INF";
	
	/**
	 * @param ctx
	 * @param usecache
	 */
	public ResourceManager(ServletContext ctx, boolean usecache) {
		this.context = ctx;
		this.usecache = usecache;
		if (usecache) {
			this.webResources = new HashMap();
			this.classPathResources = new HashMap();
		}
		if ((this.resoucesDir = ctx.getInitParameter(AjaxInitParameters.RESOURCESDIR))==null)
			this.resoucesDir = "/resources";
	}

	/**
	 * @param resource
	 * @return
	 * @throws WidgetResourceException
	 */
	public boolean isWebAppResource(String resource) throws WidgetResourceException {       
	// TODO this is the wrong place to check for external URI, since in most cases this is called it's assumed the resource is found in the classpath instead.
            if (IOUtil.isExternalUri(resource)) {
                return false;
            } else {
		return locateWebAppResource(resource)!=null;
	}
        }
        
	/**
	 * @param resource
	 * @return
	 * @throws WidgetResourceException
	 */
	public URL locateWebAppResource(String resource) throws WidgetResourceException {
		if (usecache && webResources.containsKey(resource)) {                 
			return (URL)webResources.get(resource);
		} else {
			try {
				URL url = context.getResource(resource);                           
				if (usecache)
					webResources.put(resource,url);
				return url;
			} catch (MalformedURLException mue) {
				throw new WidgetResourceException("error locating resource: '"+resource+"'",mue);
			}
		}
	}
        
	/**
	 * @param resource
	 * @return
	 */
	public boolean isClassPathResource(String resource) {
		return locateClassPathResource(resource)!=null;
	}
	
	/**
	 * @param resource
	 * @return
	 */
	public URL locateClassPathResource(String resource) {
		if (usecache && classPathResources.containsKey(resource)) {
			return (URL)classPathResources.get(resource);
		} else {
			URL url = getClass().getResource(CLASSPATHPREFIX+resource);
			if (usecache)	
				classPathResources.put(resource, url);
			return url;
		}
	}

	public String getClassPathResourcePath() {
		// TODO implement getClassPathResourcePath();
		return CLASSPATHPREFIX+getResourcesDir();
	}
	
	public String getWebResourcePath() {
		// TODO implement getWebResourcePath();
		return getResourcesDir();
	}
	
    public String getResourcesDir() {
    	return this.resoucesDir;
    }
    
    public String getWidgetDir(String widgettype) {
    	return "/"+widgettype.replace('.','/');
    }
}
