package jmaki.runtime.jsf;

import com.sun.facelets.tag.jsf.ComponentHandler;
import com.sun.facelets.tag.jsf.ComponentConfig;
import com.sun.facelets.FaceletContext;
import com.sun.facelets.tag.TagAttribute;
import javax.faces.event.ValueChangeEvent;
import javax.faces.context.FacesContext;
import com.sun.facelets.tag.MethodRule;
import com.sun.facelets.tag.MetaRuleset;

/**
 * facelethandler for jMaki's 'ajax' component
 *
 * @author Norbert von Truchsess
 */

public class AjaxWrapperHandler extends ComponentHandler {

	private TagAttribute name;

	private TagAttribute service;

	private TagAttribute selected;

	private TagAttribute template;

	private TagAttribute script;

	private TagAttribute style;
	
	private TagAttribute options;

	private TagAttribute args;

	private TagAttribute value;

	private static final MethodRule serviceRule = new MethodRule("service",String.class,new Class[] {FacesContext.class, AjaxResult.class });
	
	private static final MethodRule valueRule = new MethodRule("value",String.class,new Class[] {FacesContext.class, AjaxResult.class });  
	
	private static final MethodRule valueChangeListenerRule = new MethodRule("valueChangeListener", void.class, new Class[] { ValueChangeEvent.class });
	
	public AjaxWrapperHandler(ComponentConfig config) {
		super(config);
		name = this.getRequiredAttribute("name");
		service = this.getAttribute("service");
		selected = this.getAttribute("selected");
		template = this.getAttribute("template");
		script = this.getAttribute("script");
		style = this.getAttribute("style");
		options = this.getAttribute("options");
		args = this.getAttribute("args");
		value = this.getAttribute("value");
	}

	protected MetaRuleset createMetaRuleset(Class type) {
		return super.createMetaRuleset(type).addRule(serviceRule).addRule(valueRule).addRule(valueChangeListenerRule);
	}

	protected void setAttributes(FaceletContext ctx, Object instance) {
		super.setAttributes(ctx, instance);
		AjaxWrapper component = (AjaxWrapper) instance;
		try {
			component.setName(this.name.getValue(ctx));
			if (this.service!=null && this.service.isLiteral())
				component.setService(this.service.getValue(ctx));
			if (this.style!=null) component.setStyle(this.style.getValue(ctx));
		    if (this.script!=null) component.setScript(this.script.getValue(ctx));
		    if (this.template!=null) component.setTemplate(this.template.getValue(ctx));
		    if (this.selected!=null) component.setSelected(this.selected.getValue(ctx));
		    if (this.options!=null) component.setOptions(this.options.getValue(ctx));
		    if (this.args!=null) component.setArgs(this.args.getValue(ctx));
		    if (this.value!=null && this.value.isLiteral())
		    	component.setValue(this.value.getValue(ctx));
		} catch (Exception e) {
            throw new javax.faces.FacesException("Unable to process attributes of jMaki component: " + component.getId(),e);
		}
	}
}