package jmaki.runtime.jsf;

import javax.servlet.ServletContext;
import javax.faces.context.FacesContext;
import jmaki.runtime.AjaxContext;
import jmaki.runtime.UriManager;
import jmaki.runtime.UriManagerFactory;
import jmaki.runtime.AjaxInitParameters;
import jmaki.runtime.jsf.AjaxWrapperPhaseListener;

/**
 * @author Norbert von Truchsess
 */
public class JsfUriManagerImpl implements UriManager,UriManagerFactory {

	public String buildClassPathResourceReference(AjaxContext ctx, String resource) {
		if (ctx.getIsExtensionMapped()) {
			return ctx.getApplicationRoot()+getJsfRoot()+resource;
                }
		else {
			return ctx.getServletRoot()+resource;
                }
	}

	public String buildWebAppResourceReference(AjaxContext ctx, String resource) {
		return ctx.getApplicationRoot()+resource;
	}
	
	public String buildExternalResourceReference(AjaxContext ctx, String resource) {
		return externalRoot==null ? ctx.getWebRoot()+resource : ctx.getWebRoot()+externalRoot+resource;
	}
	
	public String buildDynamicResourceReference(AjaxContext ctx, String resource) {
            String result = null;
            FacesContext facesContext = FacesContext.getCurrentInstance();
            // PENDING(edburns): this is bogus!  I'm sick of fixing this again and
            // again.  If resource is a JSF ValueExpression.  This method
            // must return something that, when appended to the context-root
            // of this application, will cause the postback to just be
            // sent back to the faces page.
            
            if (resource.startsWith("#{") && resource.endsWith("}")) {
                String facesMapping = 
                        Util.getFacesMapping(facesContext);
                String viewId = facesContext.getViewRoot().getViewId();
                String resultPath = null;
                if (ctx.getIsExtensionMapped()) {
                    resultPath = "";
                }
                else {
                    resultPath = facesMapping + "/" + viewId;
                }
                result = ctx.getServletRoot() + resultPath;
            }
            else {
                result = ctx.getServletRoot() + 
                        resource.substring(2,resource.length()-1) +
                        AjaxWrapperPhaseListener.JMAKI_AJAX;
            }
            return result;
        }
//resource = 
//		}
//		return jsfRoot==null ? resource : ctx.getApplicationRoot()+jsfRoot+resource;
	
	public UriManager getInstance(ServletContext ctx) { 
		if (!this.isInitialized) init(ctx); 
		return this;
	}
        
	private boolean isInitialized = false;

	private void init(ServletContext ctx) {
		if (ctx.getInitParameter(AjaxInitParameters.EXTERNALROOT)!=null) {
                    externalRoot = ctx.getInitParameter(AjaxInitParameters.EXTERNALROOT);
                }
                
                // Use either a user specified JSF Root 
                // (Can't see why this would be necessary) or a heuristic.
                if (ctx.getInitParameter(AjaxInitParameters.JSFROOT)!=null) {
                    initTimeJsfRoot = ctx.getInitParameter(AjaxInitParameters.JSFROOT);
                }
                else {
                    initTimeJsfRoot = "/faces";
                }

		this.isInitialized = true;
	}
        
	private String externalRoot = null;
        
        private String initTimeJsfRoot = null;
        
        private String jsfRoot = null;
        
        private String getJsfRoot() {
            if (null == jsfRoot) {
                FacesContext facesContext = FacesContext.getCurrentInstance();
                // If we can't get a FacesContext, just use the initTime value
                if (null != facesContext) {
                    jsfRoot = initTimeJsfRoot;
                }
                else {
                    jsfRoot = Util.getFacesMapping(facesContext);
                    // If the the FacesServlet is extension mapped...
                    if (!Util.isPrefixMapped(jsfRoot)) {
                        // the jsfRoot is the empty string, and care 
                        // must be taken to append the proper mapping
                        jsfRoot = "";
                    }
                }
            }
            return jsfRoot;
        }
}
